@@ -1,5 +1,6 @@
 from requests.auth import AuthBase
 from requests.adapters import HTTPAdapter
+from requests.models import PreparedRequest
 from ntlm import ntlm
 
 
@@ -12,7 +13,7 @@ class HttpNtlmAuth(AuthBase):
             :password   - Password or hash in "ABCDABCDABCDABCD:ABCDABCDABCDABCD" format.
         """
         if ntlm is None:
-            raise Exception("NTLM libraries unavailable")        
+            raise Exception("NTLM libraries unavailable")
         #parse the username
         user_parts = username.split('\\', 1)
         self.domain = user_parts[0].upper()
@@ -21,14 +22,15 @@ class HttpNtlmAuth(AuthBase):
         self.password = password
         self.adapter = HTTPAdapter()
 
-
-    def retry_using_http_NTLM_auth(self, auth_header_field, auth_header, response):
+    def retry_using_http_NTLM_auth(self, auth_header_field, auth_header,
+                                   response, args):
         """Attempts to authenticate using HTTP NTLM challenge/response"""
 
         if auth_header in response.request.headers:
             return response
 
-        request = response.request
+        request = copy_request(response.request)
+
         # initial auth header with username. will result in challenge
         auth = 'NTLM %s' % ntlm.create_NTLM_NEGOTIATE_MESSAGE("%s\\%s" % (self.domain,self.username))
         request.headers[auth_header] = auth
@@ -36,37 +38,58 @@ class HttpNtlmAuth(AuthBase):
         # we must keep the connection because NTLM authenticates the connection, not single requests
         request.headers["Connection"] = "Keep-Alive"
 
-        response2 = self.adapter.send(request)
+        response2 = self.adapter.send(request, **args)
 
-        # this is important for some web applications that store authentication-related info in cookies (it took a long time to figure out)        
+        # this is important for some web applications that store authentication-related info in cookies (it took a long time to figure out)
         if response2.headers.get('set-cookie'):
-            headers['Cookie'] = response2.headers.get('set-cookie')
+            request.headers['Cookie'] = response2.headers.get('set-cookie')
 
         # get the challenge
         auth_header_value = response2.headers[auth_header_field]
         ServerChallenge, NegotiateFlags = ntlm.parse_NTLM_CHALLENGE_MESSAGE(auth_header_value[5:])
 
         # build response
+        request = copy_request(request)
         auth = 'NTLM %s' % ntlm.create_NTLM_AUTHENTICATE_MESSAGE(ServerChallenge, self.username, self.domain, self.password, NegotiateFlags)
         request.headers[auth_header] = auth
         request.headers["Connection"] = "Close"
 
-        response = self.adapter.send(request)
+        response3 = self.adapter.send(request, **args)
 
-        return response
+        # Update the history.
+        response3.history.append(response)
+        response3.history.append(response2)
 
+        return response3
 
-    def response_hook(self,r):
+    def response_hook(self, r, **kwargs):
 
         if r.status_code == 401 and 'ntlm' in r.headers.get('www-authenticate','').lower():
-            return self.retry_using_http_NTLM_auth('www-authenticate', 'Authorization', r)
+            return self.retry_using_http_NTLM_auth('www-authenticate',
+                                                   'Authorization', r, kwargs)
 
         if r.status_code == 407 and 'ntlm' in r.headers.get('proxy-authenticate','').lower():
-            return self.retry_using_http_NTLM_auth('proxy-authenticate', 'Proxy-authorization', r)
+            return self.retry_using_http_NTLM_auth('proxy-authenticate',
+                                                   'Proxy-authorization', r,
+                                                   kwargs)
 
         return r
 
-
-    def __call__(self,r):
+    def __call__(self, r):
         r.register_hook('response', self.response_hook)
         return r
+
+
+def copy_request(request):
+    """
+    Copies a Requests PreparedRequest.
+    """
+    new_request = PreparedRequest()
+
+    new_request.method = request.method
+    new_request.url = request.url
+    new_request.body = request.body
+    new_request.hooks = request.hooks
+    new_request.headers = request.headers.copy()
+
+    return new_request
